#ifndef HENGE_MESH_H_
#define HENGE_MESH_H_

#include "vmath.h"
#include "color.h"
#include "kdtree.h"

namespace henge {

enum {
	EL_VERTEX,
	EL_NORMAL,
	EL_TANGENT,
	EL_TEXCOORD,
	EL_INDEX,

	EL_COUNT
};

class trimesh {
private:
	Vector3 *vert;
	Vector3 *norm;
	Vector3 *tang;
	Vector2 *tc;
	unsigned int *index;
	int nvert, nnorm, ntang, ntc, nindex;

	bool dynamic;

	mutable unsigned int dlist;
	mutable unsigned int vbo[EL_COUNT];
	mutable bool vbo_valid[EL_COUNT];

	kdtree<int> kdt;
	bool kdt_valid;

	Vector3 centroid;
	Vector3 aabb_min, aabb_max;
	bool bounds_valid;

	void build_kdtree();
	void setup_vertex_arrays() const;

	void calc_bounds();

	void init();

public:
	trimesh();
	~trimesh();

	trimesh(const trimesh &mesh);
	trimesh &operator =(const trimesh &mesh);

	bool merge(const trimesh &mesh);

	/* The data pointer can be null, in which no copy is attempted.
	 * Memory allocation and internal state setup is still performed,
	 * so that afterwards data may be filled in through the pointers
	 * returned by get_data_* below.
	 */
	bool set_data(int elem, const Vector3 *data, int count);
	bool set_data(int elem, const Vector2 *data, int count);
	bool set_data(int elem, const unsigned int *data, int count);

	Vector3 *get_data_vec3(int elem);
	const Vector3 *get_data_vec3(int elem) const;

	Vector2 *get_data_vec2(int elem);
	const Vector2 *get_data_vec2(int elem) const;

	unsigned int *get_data_int(int elem);
	const unsigned int *get_data_int(int elem) const;

	int get_count(int elem) const;

	void calc_normals();

	void flip_winding();
	void flip_normals();
	void transform(const Matrix4x4 &mat);

	Vector3 get_centroid() const;
	Vector3 get_aabb_min() const;
	Vector3 get_aabb_max() const;

	void draw() const;
	void draw_normals(float sz = 1.0, const color &col = color(0, 1, 0, 1)) const;
	void draw_tangents(float sz = 1.0, const color &col = color(0, 1, 0, 1)) const;
	void draw_vertices(float sz = 1.0, const color &col = color(1, 0, 0, 1)) const;
};

}	// namespace henge

#endif	// HENGE_MESH_H_
