#include <string.h>
#include <limits.h>
#include "image.h"
#include "pixmap.h"
#include "datapath.h"

using namespace henge;

static image_file_fmt suffix_to_fmt(const char *fname);

pixmap::pixmap()
{
	xsz = ysz = -1;
	pixels = 0;
}

pixmap::pixmap(int xsz, int ysz, uint32_t *img)
{
	this->xsz = this->ysz = -1;
	set_pixels(img, xsz, ysz);
}

pixmap::~pixmap()
{
	delete [] pixels;
}

bool pixmap::set_pixels(uint32_t *img, int xsz, int ysz)
{
	if(this->xsz != xsz || this->ysz != ysz) {
		uint32_t *tmp;
		try {
			tmp = new uint32_t[xsz * ysz];
		}
		catch(...) {
			return false;
		}

		delete [] pixels;
		pixels = tmp;
		this->xsz = xsz;
		this->ysz = ysz;
	}

	if(img) {
		memcpy(pixels, img, xsz * ysz * sizeof *pixels);
	}
	return true;
}

bool pixmap::load(const char *fname)
{
	uint32_t *img;
	int xsz, ysz;

	char path[PATH_MAX];
	if(!find_file(fname, path)) {
		return false;
	}

	if(!(img = (uint32_t*)load_image(path, &xsz, &ysz))) {
		return false;
	}
	set_pixels(img, xsz, ysz);
	free_image(img);

	return true;
}

bool pixmap::save(const char *fname, image_file_fmt fmt) const
{
	if(fmt == IFMT_GUESS) {
		fmt = suffix_to_fmt(fname);
	}
	return save_image(fname, pixels, xsz, ysz, fmt);
}

bool pixmap::generate(const char *expr)
{
	/*set_pixels(0, 128, 128);	// TODO extract WxH from expr
	return texgen(pixels, xsz, ysz, expr);*/
	return false;
}

static struct {
	const char *suffix;
	image_file_fmt fmt;
} fname_suffix[] = {
	{".png", IFMT_PNG},
	{".jpg", IFMT_JPEG},
	{".jpeg", IFMT_JPEG},
	{".tga", IFMT_TGA},
	{".ppm", IFMT_PPM},
	{".rgbe", IFMT_RGBE},
	{0, (image_file_fmt)0}
};

static image_file_fmt suffix_to_fmt(const char *fname)
{
	for(int i=0; fname_suffix[i].suffix; i++) {
		const char *s;
		if((s = strstr(fname, fname_suffix[i].suffix))) {
			if(strlen(s) == strlen(fname_suffix[i].suffix)) {
				return fname_suffix[i].fmt;
			}
		}
	}
	return IFMT_PNG;	// default to png
}
