#ifndef VMATH_QUATERNION_H_
#define VMATH_QUATERNION_H_

#include <iostream>
#include "vmath_types.h"
#include "vector.h"

/* Quaternion */
class Quaternion {
public:
	scalar_t s;
	Vector3 v;

	Quaternion();
	Quaternion(scalar_t s, const Vector3 &v);
	Quaternion(scalar_t s, scalar_t x, scalar_t y, scalar_t z);
	Quaternion(const Vector3 &axis, scalar_t angle);

	Quaternion operator +(const Quaternion &quat) const;
	Quaternion operator -(const Quaternion &quat) const;
	Quaternion operator -() const;
	Quaternion operator *(const Quaternion &quat) const;
	
	void operator +=(const Quaternion &quat);
	void operator -=(const Quaternion &quat);
	void operator *=(const Quaternion &quat);

	void reset_identity();

	Quaternion conjugate() const;

	scalar_t length() const;
	scalar_t length_sq() const;
	
	void normalize();
	Quaternion normalized() const;

	Quaternion inverse() const;

	void set_rotation(const Vector3 &axis, scalar_t angle);
	void rotate(const Vector3 &axis, scalar_t angle);
	/* note: this is a totally different operation from the above
	 * this treats the quaternion as signifying direction and rotates
	 * it by a rotation quaternion by rot * q * rot'
	 */
	void rotate(const Quaternion &q);

	Matrix3x3 get_rotation_matrix() const;
	
	friend Quaternion slerp(const Quaternion &q1, const Quaternion &q2, scalar_t t);
	
	friend std::ostream &operator <<(std::ostream &out, const Quaternion &q);
};

Quaternion slerp(const Quaternion &q1, const Quaternion &q2, scalar_t t);
inline Quaternion lerp(const Quaternion &q1, const Quaternion &q2, scalar_t t);
inline Quaternion catmull_rom_spline(const Quaternion &q1, const Quaternion &q2,
		const Quaternion &q3, const Quaternion &q4, scalar_t t);

#include "quat.inl"

#endif	/* VMATH_QUATERNION_H_ */
