#include <stdio.h>
#include <setjmp.h>
#include "gxglut.h"
#include "GXBase.h"

int gxglut_app_main(int argc, char **argv);

using namespace gxbase;

static int win_width = 100, win_height = 100;
static int disp_mode;
static jmp_buf mloop_env;

static void (*disp_func)();
static void (*idle_func)();
static void (*reshape_func)(int, int);
static void (*keyb_func)(unsigned char, int, int);
static void (*keyb_up_func)(unsigned char, int, int);
static void (*mouse_func)(int, int, int, int);
static void (*motion_func)(int, int);
static void (*pasv_motion_func)(int, int);

class GXGlutWindow : public GLWindow {
private:
	int mouse_x, mouse_y;
	unsigned int bnmask;

public:
	GXGlutWindow();

	void OnDisplay();
	void OnIdle();
	void OnResize(int w, int h);
	void OnMouseButton(MouseButton button, bool down);
	void OnMouseMove(int x, int y);
	void OnKeyboard(int key, bool down);
};

class GXGlutApp : public App {
public:
	GXGlutWindow win;

	void OnCreate();
};

static GXGlutApp app;


void glutInit(int *argcp, char **argv)
{
}

void glutInitWindowSize(int w, int h)
{
	win_width = w;
	win_height = h;
}

void glutInitDisplayMode(unsigned int mode)
{
	disp_mode = mode;
}

void glutCreateWindow(const char *title)
{
	app.win.SetTitle(title);
	app.win.SetSize(win_width, win_height);
	
	app.win.SetDoubleBuffer(disp_mode & GLUT_DOUBLE);

	if(!(disp_mode & GLUT_DEPTH)) {
		app.win.SetDepthBits(0);
	}

	if(disp_mode & GLUT_STENCIL) {
		app.win.SetStencilBits(8);
	}

	if(disp_mode & GLUT_STEREO) {
		app.win.SetStereo(true);
	}

	app.win.Show();
}


void glutGameModeString(const char *mode_str)
{
	// TODO
}

void glutEnterGameMode(void)
{
	// TODO
}


void glutDisplayFunc(void (*func)(void))
{
	disp_func = func;
}

void glutIdleFunc(void (*func)(void))
{
	idle_func = func;
}

void glutReshapeFunc(void (*func)(int, int))
{
	reshape_func = func;
}

void glutKeyboardFunc(void (*func)(unsigned char, int, int))
{
	keyb_func = func;
}

void glutMouseFunc(void (*func)(int, int, int, int))
{
	mouse_func = func;
}

void glutMotionFunc(void (*func)(int, int))
{
	motion_func = func;
}


void glutMainLoop(void)
{
	if(reshape_func) {
		reshape_func(win_width, win_height);
	}

	// see comment in OnCreate for explanation
	longjmp(mloop_env, 1);
}

void glutSwapBuffers(void)
{
	app.win.SwapBuffers();
}

void glutPostRedisplay(void)
{
	app.win.Redraw();
}

void glutSolidCube(double size)
{
	double hsz = size * 0.5;

	glBegin(GL_QUADS);
	/* far face (-Z) */
	glNormal3f(0, 0, -1);
	glTexCoord2f(1, 1);	glVertex3f(hsz, -hsz, -hsz);
	glTexCoord2f(0, 1);	glVertex3f(-hsz, -hsz, -hsz);
	glTexCoord2f(0, 0);	glVertex3f(-hsz, hsz, -hsz);
	glTexCoord2f(1, 0);	glVertex3f(hsz, hsz, -hsz);
	/* top face (+Y) */
	glNormal3f(0, 1, 0);
	glTexCoord2f(0, 1); glVertex3f(-hsz, hsz, hsz);
	glTexCoord2f(1, 1); glVertex3f(hsz, hsz, hsz);
	glTexCoord2f(1, 0); glVertex3f(hsz, hsz, -hsz);
	glTexCoord2f(0, 0); glVertex3f(-hsz, hsz, -hsz);
	/* bottom face (-Y) */
	glNormal3f(0, -1, 0);
	glTexCoord2f(0, 1); glVertex3f(-hsz, -hsz, -hsz);
	glTexCoord2f(1, 1); glVertex3f(hsz, -hsz, -hsz);
	glTexCoord2f(1, 0); glVertex3f(hsz, -hsz, hsz);
	glTexCoord2f(0, 0); glVertex3f(-hsz, -hsz, hsz);
	/* right face (+X) */
	glNormal3f(1, 0, 0);
	glTexCoord2f(0, 1); glVertex3f(hsz, -hsz, hsz);
	glTexCoord2f(1, 1); glVertex3f(hsz, -hsz, -hsz);
	glTexCoord2f(1, 0); glVertex3f(hsz, hsz, -hsz);
	glTexCoord2f(0, 0); glVertex3f(hsz, hsz, hsz);
	/* left face (-X) */
	glNormal3f(-1, 0, 0);
	glTexCoord2f(0, 1); glVertex3f(-hsz, -hsz, -hsz);
	glTexCoord2f(1, 1); glVertex3f(-hsz, -hsz, hsz);
	glTexCoord2f(1, 0); glVertex3f(-hsz, hsz, hsz);
	glTexCoord2f(0, 0); glVertex3f(-hsz, hsz, -hsz);
	/* near face (+Z) */
	glNormal3f(0, 0, 1);
	glTexCoord2f(0, 1); glVertex3f(-hsz, -hsz, hsz);
	glTexCoord2f(1, 1); glVertex3f(hsz, -hsz, hsz);
	glTexCoord2f(1, 0); glVertex3f(hsz, hsz, hsz);
	glTexCoord2f(0, 0); glVertex3f(-hsz, hsz, hsz);
	glEnd();
}

// --- GXGlutApp ---
void GXGlutApp::OnCreate()
{
	int argc = ArgCount();
	char **argv = new char*[argc + 1];

	for(int i=0; i<argc; i++) {
		argv[i] = (char*)Arg(i);
	}
	argv[argc] = 0;

	/* tricky bit: At this point we need to call the app's main function to let it
	 * set glut callbacks and initialize everything. Then, when glutMainLoop is called,
	 * we need to assume control and return from OnCreate in order to continue. If on the
	 * other hand the app returns form main without calling glutMainLoop, we're supposed
	 * to exit.
	 * 
	 * So, we setjmp to save the callstack and then we call gxglut_app_main, which is
	 * actually the app's main function. If the app drops out of main, we just exit.
	 * If however the app calls glutMainLoop, that's going to longjump back here, and
	 * we can return from OnCreate and continue merrily.
	 */
	if(setjmp(mloop_env) == 0) {
		exit(gxglut_app_main(argc, argv));
	}
}

// --- GXGlutWindow ---
GXGlutWindow::GXGlutWindow()
{
	mouse_x = mouse_y = 0;
	bnmask = 0;
}

void GXGlutWindow::OnDisplay()
{
	if(disp_func) {
		disp_func();
	}
}

void GXGlutWindow::OnIdle()
{
	if(idle_func) {
		idle_func();
	}
}

void GXGlutWindow::OnResize(int w, int h)
{
	win_width = w;
	win_height = h;

	if(reshape_func) {
		reshape_func(w, h);
	}
}

void GXGlutWindow::OnMouseButton(MouseButton button, bool down)
{
	int bit = 1 << (int)button;

	if(down) {
		bnmask |= bit;
	} else {
		bnmask &= ~bit;
	}

	if(mouse_func) {
		int glut_bn = (int)button + GLUT_LEFT_BUTTON;
		mouse_func(glut_bn, down ? GLUT_DOWN : GLUT_UP, mouse_x, mouse_y);
	}
}

void GXGlutWindow::OnMouseMove(int x, int y)
{
	mouse_x = x;
	mouse_y = win_height - y;

	void (*func)(int, int) = bnmask ? motion_func : pasv_motion_func;

	if(func) {
		func(mouse_x, mouse_y);
	}
}
void GXGlutWindow::OnKeyboard(int key, bool down)
{
	void (*func)(unsigned char, int, int) = down ? keyb_func : keyb_up_func;

	if(func) {
		if(key >= 'A' && key < 'Z') {
			if(!GetAsyncKeyState(VK_SHIFT)) {
				key = tolower(key);
			}
		}
		func((unsigned char)key, mouse_x, mouse_y);
	}
}
