#include <errno.h>
#include "pthread.h"
#include "impl.h"

/* deviations: returns ESRCH if the calling thread isn't managed by this library. */
int pthread_setcancelstate(int state, int *oldstate)
{
	pthread_t tid;
	struct thread_data *t;

	if((tid = pthread_self()) == -1) {
		errno = ESRCH;
		return -1;
	}

	if(state != PTHREAD_CANCEL_ENABLE && state != PTHREAD_CANCEL_DISABLE) {
		errno = EINVAL;
		return -1;
	}

	t = _pthread_get_data(tid);

	pthread_mutex_lock(&t->mut);
	if(oldstate) {
		*oldstate = t->cancel_state;
	}
	t->cancel_state = state;
	pthread_mutex_unlock(&t->mut);
	return 0;
}

/* deviations: returns ESRCH if the calling thread isn't managed by this library. */
int pthread_setcanceltype(int type, int *oldtype)
{
	pthread_t tid;
	struct thread_data *t;

	if((tid = pthread_self()) == -1) {
		errno = ESRCH;
		return -1;
	}

	if(type != PTHREAD_CANCEL_DEFERRED && type != PTHREAD_CANCEL_ASYNCHRONOUS) {
		errno = EINVAL;
		return -1;
	}

	t = _pthread_get_data(tid);

	pthread_mutex_lock(&t->mut);
	if(oldtype) {
		*oldtype = t->cancel_type;
	}
	t->cancel_type = type;
	pthread_mutex_unlock(&t->mut);
	return 0;
}

int pthread_cancel(pthread_t tid)
{
	struct thread_data *t = _pthread_get_data(tid);
	if(!t || !t->thread) {
		errno = ESRCH;
		return -1;
	}

	SetEvent(t->cancel_event);
	return 0;
}

void pthread_testcancel(void)
{
	pthread_t tid;
	struct thread_data *t;

	if((tid = pthread_self()) == -1) {
		return;
	}

	t = _pthread_get_data(tid);

	if(t->cancel_state == PTHREAD_CANCEL_DISABLE) {
		return;
	}

	if(WaitForSingleObject(t->cancel_event, 0) == WAIT_OBJECT_0) {
		pthread_exit(0);
	}
}

