#include <errno.h>
#include "pthread.h"
#include "impl.h"

static struct tsd_dtor {
	DWORD key;
	void (*dtor)(void*);
	struct tsd_dtor *next;
} *dtor_list;

static pthread_mutex_t dtor_list_mut = PTHREAD_MUTEX_INITIALIZER;


int pthread_key_create(pthread_key_t *key, void (*destr_func)(void*))
{
	if((*key = TlsAlloc()) == TLS_OUT_OF_INDEXES) {
		errno = EAGAIN;
		return -1;
	}

	if(destr_func) {
		struct tsd_dtor *tdnode;

		if(!(tdnode = malloc(sizeof *tdnode))) {
			errno = ENOMEM;
			TlsFree(*key);
			return -1;
		}
		tdnode->key = *key;
		tdnode->dtor = destr_func;

		pthread_mutex_lock(&dtor_list_mut);
		tdnode->next = dtor_list;
		dtor_list = tdnode;
		pthread_mutex_unlock(&dtor_list_mut);
	}
	return 0;
}

int pthread_key_delete(pthread_key_t key)
{
	struct tsd_dtor dummy, *node = &dummy;
	pthread_mutex_lock(&dtor_list_mut);
	dummy.next = dtor_list;

	while(node->next) {
		if(node->next->key == key) {
			void *tmp = node->next;
			node->next = node->next->next;
			free(tmp);
			break;
		}
		node = node->next;
	}
	pthread_mutex_unlock(&dtor_list_mut);

	if(!TlsFree(key)) {
		errno = EINVAL;
		return -1;
	}
	return 0;
}

/*
void *pthread_getspecific(pthread_key_t key)
{
	return TlsGetValue(key);
}

int pthread_setspecific(pthread_key_t key, const void *ptr)
{
	if(!TlsSetValue(key, ptr)) {
		return -1;
	}
	return 0;
}
*/

#include <stdio.h>
void _pthread_tsd_cleanup(void)
{
	struct tsd_dtor *node;
	pthread_mutex_lock(&dtor_list_mut);

	node = dtor_list;
	while(node) {
		void *data = TlsGetValue(node->key);
		if(data) {
			node->dtor(data);
		}
		node = node->next;
	}
	pthread_mutex_unlock(&dtor_list_mut);
}
