#ifndef SIM_H_
#define SIM_H_

#include <list>
#include <vector>
#include "simobj.h"
#include "tpool.h"

struct BallVis {
	Vector3 pos;
	unsigned int type;
};

class Simulation {
private:
	std::list<SimBall*> ball_list;
	int num_balls;

	std::vector<SimStaticObj*> sobjects;

	Vector3 grav;
	double drag_coef;
	double timeslice, prev_upd;
	double upper_limit, lower_limit;

	void (*bounds_callback)(const SimBall*, void*);
	void *bounds_callback_cls;

	bool acc_col, acc_cont;

	unsigned long stoppage, start_pause;
	double time_scaling;

	pthread_t simloop_thread;
	bool running;

public:
	std::vector<BallVis> ball_vis;
	int num_vis_balls;
	pthread_mutex_t ballvis_mutex;

	// debugging collision and resting contact accumulators for each frame
	std::list<Contact> col_acc_list, cont_acc_list;
	
	ThreadPool tpool;


	Simulation();
	~Simulation();

	bool start();
	void stop();
	inline bool is_running() const { return running; }

	void toggle_pause();
	bool is_paused() const;

	double get_time() const;

	void set_time_scaling(double s);

	void set_collision_gathering(bool enable);
	void set_contact_gathering(bool enable);

	void set_limits(double lower, double upper);
	void get_limits(double *lower, double *upper) const;
	void set_bounds_callback(void (*func)(const SimBall*, void*), void *cls);

	void set_timeslice(double timeslice);
	double get_timeslice() const;

	void set_gravity(const Vector3 &grav);
	const Vector3 &get_gravity() const;

	void set_drag(double drag);
	double get_drag() const;

	void add_ball(SimBall *ball);
	void add_static(SimStaticObj *sobj);

	int get_ball_count() const;

	double run(double time);
	void step(double dt);
	
	int find_all_collisions(std::list<Contact> *cont_list);
	int find_collisions(SimBall *ball, std::list<Contact> *cont_list, bool append = false);
	int find_contacts(SimBall *ball, std::list<Contact> *cont_list, bool append = false);

	void update_ball_vis();
	void remove_dead_balls();

	void draw_static_objects(const henge::color &col) const;
};

#endif	// SIM_H_
